require 'fileutils'
module DeployDply
  class Venv

    attr_writer :helper

    def install(&block)
      init
      update_pip_setuptools
      yield if block_given?
      pip_install
    end

    def build(system_site_packages: false, pip_version: "8.1.2", setuptools_version: "28.7.1", &block)
      FileUtils.mkdir_p "tmp/venv"
      helper.sh "ln -sf tmp/venv -T venv"
      init
      update_pip_setuptools(pip_version, setuptools_version)
      yield if block_given?
      pip_install
      helper.sh "virtualenv-relocate -s venv"
    end

    def unpack
      helper.sh "virtualenv-relocate -d venv"
    end

    private

    def init(system_site_packages: false)
      return if File.exist? "venv/.setup_done"
      system_flag = system_site_packages ? "--system-site-packages" : ""
      helper.sh "virtualenv #{system_flag} venv"
      FileUtils.touch "venv/.setup_done"
    end

    def pip_install
      helper.sh "venv/bin/pip install -r requirements.txt"
    end

    def python(command)
      helper.sh "venv/bin/python #{command}"
    end

    def update_pip_setuptools(pip_version, setuptools_version)
      helper.sh "venv/bin/pip install pip==#{pip_version}"
      helper.sh "venv/bin/pip install setuptools==#{setuptools_version}"
      helper.sh "venv/bin/pip install urllib3[secure]"
    end

    def helper
      @helper ||= Helper.new
    end

  end
end
